@@ -24,16 +24,16 @@
 #include "ksudokugame.h"
 #include "puzzle.h"
 
-#include <qdom.h>
-//Added by qt3to4:
+#include <QDomDocument>
+#include <QFile>
 #include <QList>
 #include <QTextStream>
-#include <kurl.h>
-#include <ktemporaryfile.h>
-#include <kio/netaccess.h>
-#include <qfile.h>
-#include <klocale.h>
-#include <kstandarddirs.h>
+#include <QUrl>
+#include <QTemporaryFile>
+
+#include <KIO/FileCopyJob>
+#include <KJobWidgets>
+#include <KLocalizedString>
 
 #include "ksudoku.h"
 #include "symbols.h"
@@ -48,7 +48,7 @@ const SudokuType types[]     = {Plain, XSudoku, Jigsaw, Aztec,
 				Samurai, TinySamurai, Roxdoku,
 				Mathdoku, KillerSudoku};
 
-Game Serializer::deserializeGame(QDomElement element) {
+Game Serializer::deserializeGame(const QDomElement &element) {
 	bool hasPuzzle = false;
 	Puzzle* puzzle = 0;
 	bool hasHistory = false;
@@ -95,7 +95,7 @@ Game Serializer::deserializeGame(QDomElement element) {
 	return game;
 }
 
-Puzzle* Serializer::deserializePuzzle(QDomElement element) {
+Puzzle* Serializer::deserializePuzzle(const QDomElement &element) {
 	bool hasGraph = false;
 	SKGraph* graph = 0;
 	bool hasValues = false;
@@ -169,7 +169,7 @@ Puzzle* Serializer::deserializePuzzle(QDomElement element) {
 	return puzzle;
 }
 
-static int readInt(QDomElement element, const QString& name, int* err)
+static int readInt(const QDomElement &element, const QString& name, int* err)
 { //out of class, cannot be static
 	*err = 1;
 	QString Str = element.attribute(name);
@@ -183,7 +183,7 @@ static int readInt(QDomElement element, const QString& name, int* err)
 	return num;
 }
 
-SKGraph* Serializer::deserializeGraph(QDomElement element) {
+SKGraph* Serializer::deserializeGraph(const QDomElement &element) {
 	bool noFailure = true;
 
 	QString orderStr = element.attribute("order");
@@ -297,10 +297,10 @@ bool Serializer::deserializeClique(SKGraph * graph, const QString & size,
 	return false;
     }
 
-    QStringList  splitData = text.split(QString(" "), QString::SkipEmptyParts);
+    const QStringList  splitData = text.split(QString(" "), QString::SkipEmptyParts);
     QVector<int> data;
     data.clear();
-    foreach (QString s, splitData) {
+    for (QString s : splitData) {
 	--cellCount;
 	data << s.toInt();
 	if(cellCount <= 0) {
@@ -325,9 +325,9 @@ bool Serializer::deserializeCage(SKGraph * graph, const QDomElement & e) {
 	return false;
     }
 
-    QStringList cells = text.split(QString(" "), QString::SkipEmptyParts);
+    const QStringList cells = text.split(QString(" "), QString::SkipEmptyParts);
     cage.clear();
-    Q_FOREACH (QString s, cells) {
+    for (const QString& s : cells) {
 	cage << s.toInt();
 	size--;
 	if (size <= 0) {
@@ -339,7 +339,7 @@ bool Serializer::deserializeCage(SKGraph * graph, const QDomElement & e) {
     return true;
 }
 
-QList<HistoryEvent> Serializer::deserializeHistory(QDomElement element) {
+QList<HistoryEvent> Serializer::deserializeHistory(const QDomElement &element) {
 	QList<HistoryEvent> history;
 
 	QDomNode child = element.firstChild();
@@ -356,7 +356,7 @@ QList<HistoryEvent> Serializer::deserializeHistory(QDomElement element) {
 	return history;
 }
 
-HistoryEvent Serializer::deserializeSimpleHistoryEvent(QDomElement element) {
+HistoryEvent Serializer::deserializeSimpleHistoryEvent(const QDomElement &element) {
 	QString indexStr = element.attribute("index");
 	QString markerStr = element.attribute("markers");
 	QString valueStr = element.attribute("value");
@@ -393,34 +393,32 @@ HistoryEvent Serializer::deserializeSimpleHistoryEvent(QDomElement element) {
 	return HistoryEvent();
 }
 
-HistoryEvent Serializer::deserializeComplexHistoryEvent(QDomElement /*element*/) {
+HistoryEvent Serializer::deserializeComplexHistoryEvent(const QDomElement /*element*/&) {
 	// TODO implement this
 	return HistoryEvent();
 }
 
-SKGraph *Serializer::loadCustomShape(const KUrl &url, QWidget* window, QString *errorMsg) {
-	if ( url.isEmpty() ) return 0;
-	QString tmpFile;
-	bool success = false;
-	QDomDocument doc;
-	if(KIO::NetAccess::download(url, tmpFile, window) ) {
-		QFile file(tmpFile);
-		if(file.open(QIODevice::ReadOnly)) {
-			int errorLine;
-			if(!doc.setContent(&file, 0, &errorLine)) {
-				if(errorMsg)
-					*errorMsg = i18n("Cannot read XML file on line %1", errorLine);
-
-				return 0;
-			}
-			success = true;
-		}
-		KIO::NetAccess::removeTempFile(tmpFile);
+SKGraph *Serializer::loadCustomShape(const QUrl& url, QWidget* window, QString& errorMsg) {
+	if ( url.isEmpty() ) {
+		errorMsg = i18n("Unable to download file: URL is empty.");
+		return nullptr;
 	}
-	if ( !success ) {
-		if(errorMsg)
-			*errorMsg = i18n("Cannot load file.");
-		return 0;
+	QDomDocument doc;
+	QFile file(url.toLocalFile());
+
+	if ( !file.open(QIODevice::ReadOnly) ) {
+		errorMsg = i18n("Unable to open file.");
+		return nullptr;
+	}
+
+	int errorLine;
+	int errorColumn;
+	QString errorMessage;
+	if(!doc.setContent(&file, &errorMessage, &errorLine, &errorColumn)) {
+		qDebug() << "Error " << errorMessage << " from line " << errorLine << ":" << errorColumn << " from file " << url.toString();
+		errorMsg = i18n("Cannot read XML file on line %1", errorLine);
+
+		return nullptr;
 	}
 
 	QDomNode child = doc.documentElement().firstChild();
@@ -433,30 +431,30 @@ SKGraph *Serializer::loadCustomShape(const KUrl &url, QWidget* window, QString *
 		child = child.nextSibling();
 	}
 
-	return 0;
+	return nullptr;
 }
 
-Game Serializer::load(const KUrl& url, QWidget* window, QString *errorMsg) {
+Game Serializer::load(const QUrl& url, QWidget* window, QString& errorMsg) {
 	if ( url.isEmpty() ) return Game();
-	QString tmpFile;
-	bool success = false;
 	QDomDocument doc;
-	if(KIO::NetAccess::download(url, tmpFile, window) ) {
-		QFile file(tmpFile);
-		if(file.open(QIODevice::ReadOnly)) {
-			int errorLine;
-			if(!doc.setContent(&file, 0, &errorLine)) {
-				if(errorMsg)
-					*errorMsg = i18n("Cannot read XML file on line %1", errorLine);
-				return Game();
-			}
-			success = true;
-		}
-		KIO::NetAccess::removeTempFile(tmpFile);
+
+	QTemporaryFile tmpFile;
+	if ( !tmpFile.open() ) {
+		errorMsg = i18n("Unable to create temporary file.");
+		return Game();
 	}
-	if ( !success ) {
-		if(errorMsg)
-			*errorMsg = i18n("Cannot load file.");
+	KIO::FileCopyJob *downloadJob = KIO::file_copy(url, QUrl::fromLocalFile(tmpFile.fileName()), -1, KIO::Overwrite);
+	KJobWidgets::setWindow(downloadJob, window);
+	downloadJob->exec();
+
+	if( downloadJob->error() ) {
+		errorMsg = i18n("Unable to download file.");
+		return Game();
+	}
+
+	int errorLine;
+	if(!doc.setContent(&tmpFile, 0, &errorLine)) {
+		errorMsg = i18n("Cannot read XML file on line %1", errorLine);
 		return Game();
 	}
 
@@ -581,14 +579,14 @@ bool Serializer::serializeGraph(QDomElement &parent, const SKGraph *graph)
 	    // Add cages if this is a Mathdoku or Killer Sudoku puzzle.
 	    for (int n = 0; n < graph->cageCount(); n++) {
 		QDomElement e = parent.ownerDocument().createElement("cage");
-		QVector<int> cage = graph->cage(n);
+		const QVector<int> cage = graph->cage(n);
 		e.setAttribute("operator", graph->cageOperator(n));
 		e.setAttribute("value", graph->cageValue(n));
 		e.setAttribute("size", cage.size());
 
 		// Serialize the cell-numbers in the cage.
 		QString contentStr = " ";
-		Q_FOREACH (int cell, cage) {
+		for (const int cell : cage) {
 		    contentStr += QString::number(cell) + ' ';
 		}
 		e.appendChild(parent.ownerDocument().
@@ -682,21 +680,31 @@ bool Serializer::serializeHistoryEvent(QDomElement& parent, const HistoryEvent&
 	return true;
 }
 
-bool Serializer::store(const Game& game, const KUrl& url, QWidget* window) {
+bool Serializer::store(const Game& game, const QUrl& url, QWidget* window, QString& errorMsg) {
 	QDomDocument doc( "ksudoku" );
 	QDomElement root = doc.createElement( "ksudoku" );
 	doc.appendChild( root );
 
 	serializeGame(root, game);
 
-	KTemporaryFile file;
-	file.open();
+	QTemporaryFile file;
+	if ( !file.open() ) {
+		errorMsg = i18n("Unable to create temporary file.");
+		return false;
+	}
 
 	QTextStream stream(&file);
 	stream << doc.toString();
 	stream.flush();
 
-	KIO::NetAccess::upload(file.fileName(), url, window);
+	KIO::FileCopyJob *copyJob = KIO::file_copy(QUrl::fromLocalFile(file.fileName()), url);
+	KJobWidgets::setWindow(copyJob , window);
+	copyJob->exec();
+	if(copyJob->error())
+	{
+		errorMsg = i18n("Unable to upload file.");
+		return false;
+	}
 	return true;
 }
 
